﻿#region NameSpaces

#region System NameSpaces

using System;
using System.Net;
using System.Windows;
using System.Windows.Controls;
using System.Windows.Documents;
using System.Windows.Ink;
using System.Windows.Input;
using System.Windows.Media;
using System.Windows.Media.Animation;
using System.Windows.Shapes;
using System.Runtime.InteropServices.Automation;
using System.IO.IsolatedStorage;
using Microsoft.CSharp.RuntimeBinder;
using System.Collections.Generic;
using System.Collections.ObjectModel;

#endregion

#region Local Name Spaces

using InnovativeSolutions.SilverlightApplicationLib.Exceptions;
using InnovativeSolutions.SilverlightApplicationLib.Enumerations;

#endregion

#endregion

namespace InnovativeSolutions.SilverlightApplicationLib
{
    public class SilverlightApplication
    {
        #region Variables

        /// <summary>
        /// Flag to check whether the initialize method has been called or not.
        /// </summary>
        private Boolean _initializeFlag = false;

        /// <summary>
        /// Variable to store the temperory path of Isolated storage.
        /// </summary>
        private static string _tempISPath = string.Empty;

        /// <summary>
        /// Dictionary to store the details of Silverlight application origin uri and its corresponding folder names.
        /// </summary>
        private static Dictionary<Uri, string> _silverlightAppFolderNames;

        /// <summary>
        /// Collection Of Installed Silverlight Applications.
        /// </summary>
        private static ObservableCollection<SilverlightApplication> _silverlightApplications;

        #region  Property associated variables

        private string m_ApplicationName = string.Empty;
        private Uri m_ApplicationOrigin;
        private string m_DomainName = string.Empty;
        private string m_OOBApplicationFolderName = string.Empty;
        private string m_OOBApplicationXapPath = string.Empty;
        private string m_IsolatedStoragePath = string.Empty;
        private string m_XapName = string.Empty;

        #endregion

        #endregion

        #region Properties

        /// <summary>
        /// Gets Silverlight Application Name.
        /// </summary>
        public string ApplicationName
        {
            get
            {
                if (_initializeFlag)
                {
                    return m_ApplicationName;
                }
                else
                {
                    throw new InitializationException("Properties not Initialized. Call Initialize metod to initialize");
                }
            }
        }

        /// <summary>
        /// Gets Silverlight Application Origin Url (Url of the application's xap).
        /// </summary>
        public Uri ApplicationOrigin
        {
            get
            {
                if (_initializeFlag)
                {
                    return m_ApplicationOrigin;
                }
                else
                {
                    throw new InitializationException("Properties not Initialized. Call Initialize metod to initialize");
                }
            }
        }

        /// <summary>
        /// Gets Silverlight Application Domain Name.
        /// </summary>
        public string DomainName
        {
            get
            {
                if (_initializeFlag)
                {
                    return m_DomainName;
                }
                else
                {
                    throw new InitializationException("Properties not Initialized. Call Initialize metod to initialize");
                }
            }
        }

        /// <summary>
        /// Gets Silverlight Out Of Browser Application Folder Name.
        /// </summary>
        public string OOBApplicationFolderName
        {
            get
            {
                if (_initializeFlag)
                {
                    return m_OOBApplicationFolderName;
                }
                else
                {
                    throw new InitializationException("Properties not Initialized. Call Initialize metod to initialize");
                }
            }
        }

        /// <summary>
        /// Gets Silverlight Out Of Browser Application physical Xap path.
        /// </summary>
        public string OOBApplicationXapPath
        {
            get
            {
                if (_initializeFlag)
                {
                    return m_OOBApplicationXapPath;
                }
                else
                {
                    throw new InitializationException("Properties not Initialized. Call Initialize metod to initialize");
                }
            }
        }

        /// <summary>
        /// Gets Silverlight Application Isolated Storage Path.
        /// </summary>
        public string IsolatedStoragePath
        {
            get
            {
                if (_initializeFlag)
                {
                    return m_IsolatedStoragePath;
                }
                else
                {
                    throw new InitializationException("Properties not Initialized. Call Initialize metod to initialize");
                }
            }
        }

        /// <summary>
        /// Gets Silverlight Application Xap Name.
        /// </summary>
        public string XapName
        {
            get
            {
                if (_initializeFlag)
                {
                    return m_XapName;
                }
                else
                {
                    throw new InitializationException("Properties not Initialized. Call Initialize metod to initialize");
                }
            }
        }

        #endregion

        #region Constructor

        /// <summary>
        /// Creates a new Instance of Silverlight Application Path with Domain Name and Silverlight Application Origin Uri.
        /// </summary>
        /// <param name="originUri">Represents the Origin Url of the Silverlight Application's Xap file. </param>
        /// <param name="im">Auto : Initializes while creating Instance, Manual : Will Not Initialize, but works faster.</param>
        public SilverlightApplication(Uri originUri, InitializeMode im)
        {
            if (!Application.Current.IsRunningOutOfBrowser)
            {
                throw new RunningModeException("Application should run in Out Of Browser mode to use this class");
            }

            m_ApplicationOrigin = originUri;

            if (im == InitializeMode.Auto)
            {
                Initialize();
            }
        }

        #endregion

        #region Methods

        #region Instance Methods

        #region Initialize Section

        /// <summary>
        /// Initializes the Class Members.
        /// </summary>
        public void Initialize()
        {
            m_ApplicationName = System.IO.Path.GetFileNameWithoutExtension(m_ApplicationOrigin.ToString());
            m_DomainName = GetDomainName(m_ApplicationOrigin.ToString());

            if (new Uri(Application.Current.Host.Source.AbsoluteUri) == m_ApplicationOrigin)
            {
                m_IsolatedStoragePath = GetIsolatedStoragePath();
            }
            else
            {
                m_IsolatedStoragePath = "Not Available. Available only if it is the current application";
            }

            m_OOBApplicationFolderName = GetOOBFolderName(m_ApplicationOrigin.ToString(), m_DomainName);
            m_OOBApplicationXapPath = "Todo";
            m_XapName = System.IO.Path.GetFileName(m_ApplicationOrigin.ToString());
            _initializeFlag = true;
        }

        #endregion

        #region GetDomainName

        /// <summary>
        /// Method to get Application Domain Name.
        /// </summary>
        /// <param name="m_ApplicationOrigin">Application Origin Url.</param>
        /// <returns>Application Domain Name.</returns>
        private string GetDomainName(string appOrigin)
        {
            Boolean flag = false;
            string[] temp = appOrigin.Split(new char[] { '/' });

            foreach (string item in temp)
            {
                if (flag && item.Length > 0)
                {
                    temp[0] = item.Split(new char[] { ':' })[0];
                    break;
                }

                if (item.Contains("http"))
                {
                    flag = true;
                }
            }

            return temp[0];
        }

        #endregion

        #region GetOOBFolderName

        /// <summary>
        /// Method to get Out of Browser Folder of the Installed Silverlight Application.
        /// </summary>
        /// <param name="appOrigin">Application Origin Url.</param>
        /// <param name="domainName">Application Domain Name.</param>
        /// <returns>Application Installed Folder Name.</returns>
        private string GetOOBFolderName(string appOrigin, string domainName)
        {
            string tempFolderName = string.Empty;

            if (_silverlightAppFolderNames == null)
            {
                GetAllSilverlightApplicationFolderNames(domainName);
            }

            if (_silverlightAppFolderNames.ContainsKey(new Uri(appOrigin)))
            {
                tempFolderName = _silverlightAppFolderNames[new Uri(appOrigin)];
            }
            else
            {
                GetAllSilverlightApplicationFolderNames(domainName);
                tempFolderName = _silverlightAppFolderNames[new Uri(appOrigin)];
            }

            return tempFolderName;
        }

        #endregion

        #region Invoke Silverlight Application

        /// <summary>
        /// Invokes the associated Silverlight Application.
        /// </summary>
        public void InvokeSilverlightApp()
        {
            InvokeSilverlightApp(this);
        }

        /// <summary>
        /// Invokes the associated Silverlight Application.
        /// </summary>
        /// <param name="SllauncherPath">Path to Silverlight Application launcher (sllauncher.exe).</param>
        public void InvokeSilverlightApp(string SllauncherPath)
        {
            InvokeSilverlightApp(this, SllauncherPath);
        }

        /// <summary>
        /// Invokes the associated Silverlight Application.
        /// </summary>
        /// <param name="SllauncherPath">Path to Silverlight Application launcher (sllauncher.exe).</param>
        /// <param name="im">Specified the mode in which the application to be invoked.</param>
        public void InvokeSilverlightApp(string SllauncherPath, InvokationMode im)
        {
            InvokeSilverlightApp(this, SllauncherPath, im);
        }

        #endregion

        #region GetIsolatedStoragePath

        /// <summary>
        /// Method to Get IsolatedStorage Path.
        /// </summary>
        /// <returns>Returns the Isolated Storage path.</returns>
        private string GetIsolatedStoragePath()
        {
            string isoFileName = Guid.NewGuid().ToString();
            IsolatedStorageFile iso = IsolatedStorageFile.GetUserStoreForApplication();
            IsolatedStorageFileStream isoStream = new IsolatedStorageFileStream(isoFileName, System.IO.FileMode.Create, iso);
            isoStream.Close();

            string isFilePath = FindISFilePath(isoFileName);
            iso.DeleteFile(isoFileName);

            return isFilePath;
        }

        #endregion

        #endregion

        #region Static Methods

        #region GetInstalledSilverlightApplications

        /// <summary>
        /// Method to Get All Insalled Silverlight Applications.
        /// </summary>
        /// <returns>Collection of Installed Silverlight Applications.</returns>
        public static ObservableCollection<SilverlightApplication> GetInstalledSilverlightApplications()
        {
            if (_silverlightApplications == null)
            {
                _silverlightApplications = new ObservableCollection<SilverlightApplication>();
            }

            _silverlightApplications.Clear();
            Dictionary<Uri, string> tempDict = GetAllSilverlightApplicationFolderNames();

            foreach (Uri item in tempDict.Keys)
            {
                SilverlightApplication obj = new SilverlightApplication(item, InitializeMode.Auto);
                _silverlightApplications.Add(obj);
            }

            return _silverlightApplications;
        }

        /// <summary>
        /// Method to Get Collection of Installed Silverligt applications belonging to a particular domain.
        /// </summary>
        /// <param name="domainNames">List of domains to check with.</param>
        /// <returns>Collection of Installed Silverlight Applications in a particular Domains.</returns>
        public static ObservableCollection<SilverlightApplication> GetInstalledSilverlightApplications(string[] domainNames)
        {
            if (_silverlightApplications == null)
            {
                _silverlightApplications = new ObservableCollection<SilverlightApplication>();
            }

            _silverlightApplications.Clear();
            Dictionary<Uri, string> tempDict = GetAllSilverlightApplicationFolderNames(domainNames);

            foreach (Uri item in tempDict.Keys)
            {
                SilverlightApplication obj = new SilverlightApplication(item, InitializeMode.Manual);
                _silverlightApplications.Add(obj);
            }

            return _silverlightApplications;
        }

        #endregion

        #region GetCurrentApplicationOriginUri

        /// <summary>
        /// Returns Current Silverlight Application Origin Uri.
        /// </summary>
        /// <returns>Application Origin Url.</returns>
        public static Uri GetCurrentApplicationOriginUri()
        {
            return new Uri(Application.Current.Host.Source.AbsoluteUri);
        }

        #endregion

        #region GetAllSilverlightApplicationFolderNames

        /// <summary>
        /// Method to Get all Silverlight Applications Folder Name and Origin uri for the provided domains.
        /// </summary>
        /// <param name="domainNames">Array of domains to check with.</param>
        /// <returns>Resulting Dictionary of Folder Names and Origin Uri.</returns>
        private static Dictionary<Uri, string> GetAllSilverlightApplicationFolderNames(string[] domainNames)
        {
            if (_silverlightAppFolderNames == null)
            {
                _silverlightAppFolderNames = new Dictionary<Uri, string>();
            }

            _silverlightAppFolderNames.Clear();

            foreach (string item in domainNames)
            {
                GetAllSilverlightApplicationFolderNames(item);
            }

            return _silverlightAppFolderNames;
        }

        /// <summary>
        /// Method to Get All Installed Silverlight Applications FolderName and Origin Uri.
        /// </summary>
        /// <returns>Resulting Dictionary of Folder Names and Origin Uri.</returns>
        private static Dictionary<Uri, string> GetAllSilverlightApplicationFolderNames()
        {
            if (_silverlightAppFolderNames == null)
            {
                _silverlightAppFolderNames = new Dictionary<Uri, string>();
            }

            _silverlightAppFolderNames.Clear();

            // Todo : Get Files in Index Folder.
            throw new NotImplementedException();

            return _silverlightAppFolderNames;
        }

        /// <summary>
        /// Gets the Folder Names and application Origins of installed Silverlight application under a particular domain.
        /// </summary>
        /// <param name="domainName">Application Domain Name.</param>
        private static void GetAllSilverlightApplicationFolderNames(string domainName)
        {
            if (_silverlightAppFolderNames == null)
            {
                _silverlightAppFolderNames = new Dictionary<Uri, string>();
            }

            string indexFilePath = GetIndexFilePath(domainName);
            dynamic fso = AutomationFactory.CreateObject("Scripting.FileSystemObject");

            try
            {
                dynamic IndexContentsStream = fso.OpenTextFile(indexFilePath);
                string strTemp = string.Empty;

                while (true)
                {
                    strTemp = IndexContentsStream.ReadLine();
                    if (strTemp == "\0")
                    {
                        break;
                    }
                    else
                    {
                        AddToFolderNameDictionary(strTemp, domainName);
                    }
                }
            }
            catch (Exception ex)
            {
                throw new InvalidDomainException("Please check the Domain Name Given : " + ex.Message);
            }
        }

        /// <summary>
        /// Method to Get Index File Path.
        /// </summary>
        /// <param name="domainName">Application Domain Name.</param>
        /// <returns>Index File Path.</returns>
        private static string GetIndexFilePath(string domainName)
        {
            string indexFilePath = string.Empty;
            indexFilePath = System.IO.Path.GetDirectoryName(Environment.GetFolderPath(Environment.SpecialFolder.MyDocuments));
            indexFilePath = indexFilePath + "\\AppData\\LocalLow\\Microsoft\\Silverlight\\OutOfBrowser\\index\\" + domainName;
            return indexFilePath;
        }

        /// <summary>
        /// Method to add details to the Silverlight Application folder names Dictionary.
        /// </summary>
        /// <param name="tempRecord">String from which the details are extracted.</param>
        /// <param name="domainName">Application Domain Name.</param>
        private static void AddToFolderNameDictionary(string tempRecord, string domainName)
        {
            tempRecord = ClearControlCharactersAndWhiteSpaces(tempRecord);
            string key = GetApplicationUriFromRecord(tempRecord);
            string value = GetFolderNumberFromRecord(tempRecord) + "." + domainName;

            if (!_silverlightAppFolderNames.ContainsKey(new Uri(key)))
            {
                _silverlightAppFolderNames.Add(new Uri(key), value);
            }
        }

        /// <summary>
        /// Method to clear Control Characters and Whitespaces from String.
        /// </summary>
        /// <param name="tempRecord">String from which control character and white space to be removed.</param>
        /// <returns>Filtered string.</returns>
        private static string ClearControlCharactersAndWhiteSpaces(string tempRecord)
        {
            string temp = string.Empty;

            foreach (char i in tempRecord.ToCharArray())
            {
                if (!char.IsControl(i) && !char.IsWhiteSpace(i))
                {
                    temp = temp + i;
                }
            }

            return temp;
        }

        /// <summary>
        /// Method to get the Application Uri from Record.
        /// </summary>
        /// <param name="tempRecord">String from which the details are extracted.</param>
        /// <returns>Application Uri.</returns>
        private static string GetApplicationUriFromRecord(string tempRecord)
        {
            string temp = string.Empty;

            foreach (char i in tempRecord.ToCharArray())
            {
                if (!char.IsDigit(i))
                {
                    temp = tempRecord.Remove(0, tempRecord.IndexOf(i));
                    break;
                }
            }

            return temp;
        }

        /// <summary>
        /// Method to get the Url of the Silverlight application Number from the record.
        /// </summary>
        /// <param name="tempRecord">String from which the details are extracted.</param>
        /// <returns>Silverlight Application Number.</returns>
        private static string GetFolderNumberFromRecord(string tempRecord)
        {
            string temp = string.Empty;

            foreach (char i in tempRecord.ToCharArray())
            {
                if (char.IsDigit(i))
                {
                    temp = temp + i;
                }
                else
                {
                    break;
                }
            }

            return temp;
        }

        #endregion

        #region InvokeSilverlightApp

        /// <summary>
        /// Invokes the Specified Silverlight Application in Out Of Browser Mode.
        /// </summary>
        /// <param name="SLApplication">Silverlight Application to Invoke.</param>
        public static void InvokeSilverlightApp(SilverlightApplication SLApplication)
        {
            string slLauncherPath = "C:\\Program Files\\Microsoft Silverlight\\sllauncher.exe";
            InvokeSilverlightApp(SLApplication, slLauncherPath);
        }

        /// <summary>
        /// Invokes the Specified Silverlight Application in Out Of Browser Mode.
        /// </summary>
        /// <param name="SLApplication">Silverlight Application to Invoke.</param>
        /// <param name="SllauncherPath">Path to Silverlight Application launcher (sllauncher.exe).</param>
        public static void InvokeSilverlightApp(SilverlightApplication SLApplication, string SllauncherPath)
        {
            InvokeSilverlightApp(SLApplication, SllauncherPath, InvokationMode.Emulate);
        }

        /// <summary>
        /// Invokes the Specified Silverlight Application in Out Of Browser Mode.
        /// </summary>
        /// <param name="SLApplication">ilverlight Application to Invoke.</param>
        /// <param name="SllauncherPath">Path to Silverlight Application launcher (sllauncher.exe).</param>
        /// <param name="im">Specified the mode in which the application to be invoked.</param>
        public static void InvokeSilverlightApp(SilverlightApplication SLApplication, string SllauncherPath, InvokationMode im)
        {
            // Todo : Invoking of Silverlight Application
            throw new NotImplementedException();
        }

        #endregion

        #region FindISFilePath

        /// <summary>
        /// Method to find the Path of Given Isolated File.
        /// </summary>
        /// <param name="fileName"></param>
        /// <returns></returns>
        private static string FindISFilePath(string fileName)
        {
            string isFilePath = string.Empty;
            string rootFolder = System.IO.Path.GetDirectoryName(Environment.GetFolderPath(Environment.SpecialFolder.MyDocuments)) + "\\AppData\\LocalLow\\Microsoft\\Silverlight\\is\\";
            dynamic fso = AutomationFactory.CreateObject("Scripting.FileSystemObject");
            dynamic fldr = fso.GetFolder(rootFolder);

            GetISFilePathInRootFolder(fldr, fileName);
            isFilePath = _tempISPath;

            return isFilePath;
        }

        /// <summary>
        /// Gets the path of IS storage File by recursively calling itself and stores it to a temperory variable.
        /// </summary>
        /// <param name="folderPath">Path of Root Folder.</param>
        /// <param name="fileName">Isolated Storage File Name.</param>
        private static void GetISFilePathInRootFolder(dynamic folder, string fileName)
        {
            dynamic fso = AutomationFactory.CreateObject("Scripting.FileSystemObject");

            if (!fso.FileExists(folder.Path + "\\" + fileName))
            {
                foreach (dynamic fldr in folder.Subfolders)
                {
                    GetISFilePathInRootFolder(fldr, fileName);
                }
            }
            else
            {
                _tempISPath = folder.Path;
            }
        }

        #endregion

        #endregion

        #endregion
    }
}
